( function( $, settings ) {

	var onDialogButton = happyForms.dashboard.onDialogButton;

	happyForms.dashboard.onDialogButton = function( e ) {
		e.preventDefault();
		e.stopImmediatePropagation();

		var formId = $( '#happyforms-dialog-select' ).val();

		if ( ! formId ) {
			return false;
		}

		if ( 1 == settings.forms[formId].modal ) {
			var link = settings.modalLink.replace( 'ID', formId );

			window.parent.send_to_editor( link );
			$( '#happyforms-modal' ).dialog( 'close' );
			$( '#happyforms-dialog-select' ).val( '' );

			if ( editor = this.getCurrentEditor() ) {
				editor.focus();
			}

			return;
		}

		return onDialogButton.apply( this, arguments );
	}

	var ResendUserEmail = function() {
		this.$link = null;
		this.timeout = null;
		this.interval = 5000;
	}

	ResendUserEmail.prototype.bind = function() {
		this.$link = $( 'a.happyforms-resend-user-email' );
		this.$link.click( this.onClick.bind( this ) );
	}

	ResendUserEmail.prototype.onClick = function( e ) {
		e.preventDefault();
		e.stopImmediatePropagation();

		var responseID = this.$link.attr( 'data-response-id' );

		if ( this.timeout ) {
			return;
		}

		this.send( responseID );
		this.timeout = setTimeout( this.reset.bind( this ), this.interval );
	}

	ResendUserEmail.prototype.send = function( responseID ) {
		var url = this.$link.attr( 'href' );

		$.post( url, function() {
			console.log( arguments );
		} );
	}

	ResendUserEmail.prototype.reset = function( responseID ) {
		clearTimeout( this.timeout );
		this.timeout = null;
	}

	happyForms.dashboard.focusFirstResponseInput = function() {
		$( '.happyforms-edit-message-table input[type="text"],' +
		   ' .happyforms-edit-message-table input[type="number"],' +
		   ' .happyforms-edit-message-table textarea' ).first().focus();
	}

	happyForms.dashboard.handleEditActivityMetaboxes = function() {
		var $sortables = $( '.post-type-happyforms-message #side-sortables' );

		if ( ! $sortables.length ) {
			return;
		}

		$sortables.sortable( 'destroy' );

		$( '.post-type-happyforms-message #submitdiv' ).removeClass( 'postbox' ).addClass( 'stuffbox' );
	}

	happyForms.dashboard.animateTableRow = function( $tr, colorFrom, colorTo ) {
		$tr.animate( { 'backgroundColor': colorFrom }, 'fast' )
			.animate( { backgroundColor: colorTo }, {
				complete: function() {
					$tr.css( 'backgroundColor', '' );
				}
			} );
	}

	var ActivityRowActions = function() {
		this.$link = null;
	}

	ActivityRowActions.prototype.bind = function() {
		this.$link = $( '.mark_not_spam a, .untrash a' );
		this.$link.click( this.onClick.bind( this ) );
	}

	ActivityRowActions.prototype.onClick = function( e ) {
		e.preventDefault();
		e.stopImmediatePropagation();

		window.location = $( e.target ).attr( 'href' );

		var $tr = $( e.target ).parents( 'tr' );
		$tr.removeClass( 'happyforms-message-unread' );
		happyForms.dashboard.animateTableRow( $tr, '#e7e7d3', 'rgba(249, 249, 249)' );
	}

	var FormTable = function() {
		this.$link = null;
	}

	FormTable.prototype.bind = function() {
		this.settings = ( 'undefined' !== typeof _happyFormsFormStatusSettings ) ? _happyFormsFormStatusSettings : false;
		this.$table = $( 'body.post-type-happyform .wp-list-table' );
		this.$restoreLink = $( 'body.post-type-happyform .restore a' );

		if ( this.$restoreLink.length ) {
			this.$restoreLink.click( this.onRestoreClick.bind( this ) );
		}

		this.rewordNoFormsInArchive();
		this.removeLinksFromArchivedForms();
	}

	FormTable.prototype.onRestoreClick = function( e ) {
		e.preventDefault();
		e.stopImmediatePropagation();

		window.location = $( e.target ).attr( 'href' );

		var $tr = $( e.target ).parents( 'tr' );
		happyForms.dashboard.animateTableRow( $tr, '#e7e7d3', 'rgba(249, 249, 249)' );
	}

	FormTable.prototype.removeLinksFromArchivedForms = function() {
		$( 'tr.type-happyform.status-archive' ).each( function() {
			var $this = $( this );
			var $linkWrap = $( '.column-title > strong', $this );
			var linkText = $( 'a.row-title', $this ).text();

			$( 'a.row-title', $this ).remove();
			$linkWrap.text( linkText );
		} );
	}

	FormTable.prototype.rewordNoFormsInArchive = function() {
		if ( ! this.settings ) {
			return;
		}

		$( 'tr.no-items td', this.$table ).text( this.settings.labels.not_found_in_archive );
	}

	$( document ).ready( function() {
		happyForms.dashboard.resendUserEmail = new ResendUserEmail();
		happyForms.dashboard.resendUserEmail.bind();
		happyForms.dashboard.focusFirstResponseInput();
		happyForms.dashboard.handleEditActivityMetaboxes();

		happyForms.dashboard.activityRowActions = new ActivityRowActions();
		happyForms.dashboard.activityRowActions.bind();

		happyForms.dashboard.formTable = new FormTable();
		happyForms.dashboard.formTable.bind();
	} );

} )( jQuery, _happyFormsAdmin );
