<?php
class HappyForms_Conditionals_UI_Controller {

	private static $instance;

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		add_filter( 'happyforms_setup_controls', array( $this, 'append_control_settings' ) );
		add_filter( 'happyforms_email_controls', array( $this, 'append_control_settings' ) );
		add_action( 'happyforms_customize_enqueue_scripts', array( $this, 'customize_enqueue_scripts' ) );
		add_action( 'customize_controls_print_footer_scripts', array( $this, 'print_customize_templates' ) );
		add_filter( 'happyforms_customize_part_logic_template_path', array( $this, 'set_part_logic_template_path' ) );
	}

	public function set_part_logic_template_path( $path ) {
		$path = happyforms_get_include_folder() . '/templates/customize-form-part-logic.php';

		return $path;
	}

	public function get_supported_part_types() {
		$part_library = happyforms_get_part_library();
		$parts = $part_library->get_parts();

		return apply_filters( 'happyforms_conditionals_supported_parts', array_keys( $parts ) );
	}

	public function get_enabled_controls() {
		$supported_controls = array(
			'email_recipient' => array(
				'type' => 'set',
				'then_text' => __( 'Then email address is…', 'happyforms' )
			),
			'email_bccs' => array(
				'type' => 'set',
				'then_text' => __( 'Then email address is…', 'happyforms' )
			),
			'confirmation_email_subject' => array(
				'type' => 'set',
				'then_text' => __( 'Then subject…', 'happyforms' )
			),
			'redirect_url' => array(
				'type' => 'set',
				'then_text' => __( 'Paste web address or type to search', 'happyforms' )
			)
		);

		return apply_filters( 'happyforms_conditional_enabled_setup_controls', $supported_controls );
	}

	public function append_control_settings( $controls ) {
		$conditionals = $this->get_enabled_controls();
		$field_controls = array_filter( $controls, function( $control ) {
			return isset( $control['field'] );
		} );
		$control_keys = wp_list_pluck( $field_controls, 'field' );
		$control_keys = array_flip( $control_keys );
		$control_keys = array_intersect_key( $control_keys, $conditionals );

		foreach( $control_keys as $field => $key ) {
			$controls[$key]['conditional_settings'] = $conditionals[$field];
			add_action( "happyforms_setup_control_{$field}_after", array( $this, 'customize_render_setup_logic_template' ) );
		}

		return $controls;
	}

	public function customize_render_setup_logic_template( $control ) {
		require( happyforms_get_include_folder() . '/templates/customize-form-setup-logic.php' );
	}

	public function print_customize_templates() {
		require_once( happyforms_get_include_folder() . '/templates/customize-form-logic-group.php' );
		require_once( happyforms_get_include_folder() . '/templates/customize-form-logic-item.php' );
	}

	public function customize_enqueue_scripts() {
		wp_enqueue_script(
			'happyforms-conditionals',
			happyforms_get_plugin_url() . 'inc/assets/js/customize/logic.js',
			array( 'happyforms-customize' ),
			HAPPYFORMS_UPGRADE_VERSION,
			true
		);

		$controls = $this->get_enabled_controls();

		wp_localize_script(
			'happyforms-conditionals',
			'_happyFormsConditionSettings',
			array(
				'constants' => HappyForms_Condition::get_constants(),
				'controls' => $controls
			)
		);
	}

}

if ( ! function_exists( 'happyforms_get_conditionals_ui_controller' ) ):

function happyforms_get_conditionals_ui_controller() {
	return HappyForms_Conditionals_UI_Controller::instance();
}

endif;

happyforms_get_conditionals_ui_controller();
