<?php

class HappyForms_Logger {

	private static $instance;

	public $cookie_name = 'happyforms_logger';
	public $comment_type = 'happyforms-event';

	const EVENT_ERROR = 'error';
	const EVENT_SUBMIT = 'submit';
	const EVENT_LEAD = 'lead';

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		add_action( 'send_headers', array( $this, 'set_tracking_id' ) );
		add_action( 'happyforms_validation_error', array( $this, 'log_error' ) );
		add_action( 'happyforms_submission_success', array( $this, 'log_submission' ), 10, 2 );
		add_action( 'pre_get_comments', array( $this, 'hide_logs' ) );
	}

	public function generate_tracking_id() {
		$ip = happyforms_get_client_ip();
		$ua = happyforms_get_client_user_agent();
		$time = time();
		$uid = wp_generate_uuid4();
		$id = md5( "{$uid}_{$ip}_{$ua}_{$time}" );

		return $id;
	}

	public function is_enabled() {
		$enabled = apply_filters( 'happyforms_logger_enabled', true );

		return $enabled;
	}

	public function set_tracking_id() {
		if ( ! $this->is_enabled() ) {
			return;
		}

		if ( isset( $_COOKIE[$this->cookie_name] ) ) {
			return;
		}

		$id = $this->generate_tracking_id();
		setcookie( $this->cookie_name, $id, 0, COOKIEPATH, COOKIE_DOMAIN );
	}

	public function get_tracking_id() {
		if ( ! isset( $_COOKIE[$this->cookie_name] ) ) {
			return $this->generate_tracking_id();
		}

		return $_COOKIE[$this->cookie_name];
	}

	public function log( $event, $form_id = '', $unique = true ) {
		$user_id = $this->get_tracking_id();
		$event_id = $this->get( $event, $user_id, $form_id );
		$date = current_time( 'mysql' );
		$date_gmt = get_gmt_from_date( $date );
		$data = array(
			'comment_post_ID' => $form_id,
			'comment_author' => $user_id,
			'comment_type' => $this->comment_type,
			'comment_approved' => 9,
			'comment_date' => $date,
			'comment_date_gmt' => $date_gmt,
			'comment_content' => $event,
		);

		if ( ! is_null( $event_id ) ) {
			$data['comment_ID'] = $event_id;
			wp_update_comment( $data );
		} else {
			wp_insert_comment( $data );
		}
	}

	public function log_error( $form ) {
		if ( ! $this->is_enabled() ) {
			return;
		}

		$form_id = $form['ID'];
		$this->log( self::EVENT_ERROR, $form_id );
	}

	public function log_submission( $submission, $form ) {
		if ( ! $this->is_enabled() ) {
			return;
		}

		$form_id = $form['ID'];

		foreach( $submission as $part_id => $value ) {
			if ( false === strpos( $part_id, 'email' ) ) {
				continue;
			}

			if ( $this->is_new_email( $value ) ) {
				$this->log( self::EVENT_LEAD, $form_id );
			}
		}

		$this->log( self::EVENT_SUBMIT, $form_id );
	}

	public function get( $event, $user_id, $form_id ) {
		global $wpdb;

		$query = "
			SELECT comment_ID
			FROM $wpdb->comments
			WHERE comment_type = '$this->comment_type'
			AND comment_content = '$event'
		";

		if ( ! empty( $user_id ) ) {
			$query .= "AND comment_author = '$user_id'";
		}

		if ( ! empty( $form_id ) ) {
			$query .= "AND comment_post_ID = $form_id";
		}

		$id = $wpdb->get_var( $query );

		return $id;
	}

	public function is_new_email( $email ) {
		global $wpdb;

		$post_type = happyforms_get_message_controller()->post_type;

		$query = "
			SELECT COUNT(p.ID)
			FROM $wpdb->posts p
			JOIN $wpdb->postmeta m
			ON p.ID = m.post_id
			AND p.post_type = '$post_type'
			AND m.meta_key LIKE '_happyforms_email%'
			AND m.meta_value = '$email'
			GROUP BY m.meta_value
		";

		$count = $wpdb->get_var( $query );
		$is_new = '1' === "$count";

		return $is_new;
	}

	public function hide_logs( $query ) {
		if ( $this->comment_type !== $query->query_vars['type'] ) {
			$query->query_vars['type__not_in'] = array_merge(
				( array ) $query->query_vars['type__not_in'],
				array( $this->comment_type )
			);
		}
	}

}

if ( ! function_exists( 'happyforms_get_logger' ) ):

function happyforms_get_logger() {
	return HappyForms_Logger::instance();
}

endif;

happyforms_get_logger();
