<?php

class HappyForms_Message_Blocklist {

	private static $instance;
	private static $hooked = false;

	public $save_action = 'happyforms_save_message_blocklist';
	public $save_nonce = 'happyforms-validation-message-blocklist';
	public $option = 'happyforms_blocklist_keys';

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		if ( self::$hooked ) {
			return;
		}

		self::$hooked = true;

		add_action( 'wp_ajax_' . $this->save_action, array( $this, 'save_settings' ) );
	}

	public function get_keys( $as_array = false ) {
		$keys = get_option( $this->option, '' );
		$keys = wp_unslash( $keys );

		if ( $as_array ) {
			$keys = explode( "\n", $keys );
			$keys = array_map( 'trim', $keys );
			$keys = array_filter( $keys );
		}

		return $keys;
	}

	public function update_keys( $keys ) {
		$keys = explode( "\n", $keys );
		$keys = array_map( 'trim', $keys );
		$keys = array_filter( $keys );
		$keys = implode( "\n", $keys );

		update_option( $this->option, $keys );
	}

	public function save_settings() {
		if ( ! check_ajax_referer( $this->save_action, $this->save_nonce ) ) {
			return;
		}

		$keys = isset( $_REQUEST['blocklist_keys'] ) ? $_REQUEST['blocklist_keys'] : '';
		$this->update_keys( $keys );

		ob_start();
		require_once( happyforms_get_include_folder() . '/templates/admin-settings-blocklist.php' );
		$response = ob_get_clean();

		wp_send_json_success( array(
			'html' => $response,
			'message' => __( 'Changes saved.', 'happyforms' ),
		) );
	}

	public function includes_payment( $form, $request ) {
		$payment_part = happyforms_get_form_controller()->get_first_part_by_type( $form, 'payments' );

		if ( ! $payment_part ) {
			return;
		}

		$payment_part_id = $payment_part['id'];
		$payment_part_name = happyforms_get_part_name( $payment_part, $form );
		$payment_part_value = $request[$payment_part_name];
		$price = isset( $payment_part_value['price'] ) ? $payment_part_value['price'] : '';
		$price = floatval( $price );
		$includes_payment = $price > 0;

		return $includes_payment;
	}

	public function check_message( $message, $form, $request ) {
		$is_spam = false;

		/*
		if ( $this->includes_payment( $form, $request ) ) {
			return $is_spam;
		}

		$keys = $this->get_keys( true );
		$content = implode( "\n", array_values( $message['parts'] ) );
		$content = wp_unslash( $content );
		$content_without_html = wp_strip_all_tags( $content );
		$content_without_html = wp_unslash( $content_without_html );
		$ip = happyforms_get_client_info()->get_ip();

		foreach( $keys as $key ) {
			$key = preg_quote( $key, '#' );
			$pattern = "#($|\s|\b)$key(^|\s|\b)#imu";

			if ( preg_match( $pattern, $content )
				|| preg_match( $pattern, $content_without_html )
				|| ( ! empty( $ip ) && preg_match( $pattern, $ip ) ) ) {

				$is_spam = true;
				break;
			}
		}

		if ( $is_spam ) {
			happyforms_update_meta( $message['ID'], 'read', 2 );
			happyforms_get_message_controller()->update_counters();
		}
		*/

		return $is_spam;
	}

}

if ( ! function_exists( 'happyforms_get_message_blocklist' ) ):

function happyforms_get_message_blocklist() {
	return HappyForms_Message_Blocklist::instance();
}

endif;

happyforms_get_message_blocklist();
