<?php
if ( ! function_exists( 'happyforms_upgrade_template_path' ) ) :

function happyforms_upgrade_template_path( $path, $form = array(), $part = array() ) {
	$path = apply_filters( 'happyforms_get_upgrade_template_path', $path, $form, $part );
	$path = "/templates/{$path}.php";
	$path = happyforms_get_include_folder() . $path;

	return $path;
}

endif;

if ( ! function_exists( 'happyforms_password_submit' ) ) :

function happyforms_password_submit( $form ) {
	include( happyforms_upgrade_template_path( 'partials/form-password-submit', $form ) );
}

endif;

if ( ! function_exists( 'happyforms_get_client_ip' ) ) :

function happyforms_get_client_ip() {
	$keys = array( 'HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR' );

	foreach( $keys as $key ) {
		if ( isset( $_SERVER[$key] ) && ! empty( $_SERVER[$key] ) ) {
			return $_SERVER[$key];
		}
	}

	return '';
}

endif;

if ( ! function_exists( 'happyforms_get_client_user_agent' ) ) :

function happyforms_get_client_user_agent() {
	$key = 'HTTP_USER_AGENT';

	if ( isset( $_SERVER[$key] ) && ! empty( $_SERVER[$key] ) ) {
		return $_SERVER[$key];
	}

	return '';
}

endif;

if ( ! function_exists( 'happyforms_get_current_url' ) ) :

function happyforms_get_current_url() {
	$protocol = isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] !== 'off' ? 'https' : 'http';
	$host = $_SERVER['HTTP_HOST'];
	$uri = $_SERVER['REQUEST_URI'];
	$url = $protocol . '://' . $host . $uri;

	return $url;
}

endif;

if ( ! function_exists( 'happyforms_get_referer' ) ) :

function happyforms_get_referer() {
	$key = 'HTTP_REFERER';

	if ( isset( $_SERVER[$key] ) && ! empty( $_SERVER[$key] ) ) {
		return $_SERVER[$key];
	}

	return '';
}

endif;

if ( ! function_exists( 'happyforms_form_is_available' ) ) :

function happyforms_form_is_available( $form ) {
	$available = true;

	$available = $available && ( ! happyforms_get_schedule()->is_restricted( $form ) );

	return $available;
}

endif;

if ( ! function_exists( 'happyforms_get_page_breaks' ) ) :

function happyforms_get_page_breaks( $form ) {
	$breaks = happyforms_get_form_controller()->get_parts_by_type( $form, 'page_break' );
	$breaks = wp_list_pluck( $breaks, 'id' );

	return $breaks;
}

endif;

if ( ! function_exists( 'happyforms_get_current_page_break' ) ) :

function happyforms_get_current_page_break( $form, $index = false ) {
	$step = happyforms_get_current_step( $form );
	$breaks = happyforms_get_page_breaks( $form );

	if ( ! in_array( $step, $breaks ) ) {
		return false;
	}

	if ( true === $index ) {
		$step = array_search( $step, $breaks );
	}

	return $step;
}

endif;


if ( ! function_exists( 'happyforms_get_poll_votes_label' ) ) :

function happyforms_get_poll_votes_label( $votes_number ) {
	$label = __( 'votes', 'happyforms' );

	if ( 1 === intval( $votes_number ) ) {
		$label = __( 'vote', 'happyforms' );
	}

	return $label;
}

endif;

if ( ! function_exists( 'happyforms_submit_enabled' ) ) :

function happyforms_submit_enabled( $form ) {
	$disabled = happyforms_get_form_property( $form, 'disable_submit_until_valid' );
	$preview = happyforms_is_preview_context();
	return $disabled || $preview;
}

endif;

if ( ! function_exists( 'happyforms_get_part_choice_letter' ) ) :

function happyforms_get_part_choice_letter( $option, $part ) {
	$letters = range( 'A', 'Z' );

	if ( ! empty( $option ) ) {
		$option_index = array_search( $option['id'], array_column( $part['options'], 'id' ) );
		$letter = $letters[$option_index];
	} else {
		$options_count = count( $part['options'] );
		$letter = $letters[$options_count];
	}

	return $letter;
}

endif;

if ( ! function_exists( 'happyforms_geolocation_button' ) ) :

function happyforms_geolocation_button( $part ) {
	?>
	<button class="happyforms-plain-button happyforms-address-geolocate happyforms-address-geolocate--default">
		<span class="screen-reader-text"><?php _e( 'Get location', 'happyforms' ); ?></span>
		<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="crosshairs" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" class="happyforms-address-geolocate__crosshair"><path fill="currentColor" d="M500 224h-30.364C455.724 130.325 381.675 56.276 288 42.364V12c0-6.627-5.373-12-12-12h-40c-6.627 0-12 5.373-12 12v30.364C130.325 56.276 56.276 130.325 42.364 224H12c-6.627 0-12 5.373-12 12v40c0 6.627 5.373 12 12 12h30.364C56.276 381.675 130.325 455.724 224 469.636V500c0 6.627 5.373 12 12 12h40c6.627 0 12-5.373 12-12v-30.364C381.675 455.724 455.724 381.675 469.636 288H500c6.627 0 12-5.373 12-12v-40c0-6.627-5.373-12-12-12zM288 404.634V364c0-6.627-5.373-12-12-12h-40c-6.627 0-12 5.373-12 12v40.634C165.826 392.232 119.783 346.243 107.366 288H148c6.627 0 12-5.373 12-12v-40c0-6.627-5.373-12-12-12h-40.634C119.768 165.826 165.757 119.783 224 107.366V148c0 6.627 5.373 12 12 12h40c6.627 0 12-5.373 12-12v-40.634C346.174 119.768 392.217 165.757 404.634 224H364c-6.627 0-12 5.373-12 12v40c0 6.627 5.373 12 12 12h40.634C392.232 346.174 346.243 392.217 288 404.634zM288 256c0 17.673-14.327 32-32 32s-32-14.327-32-32c0-17.673 14.327-32 32-32s32 14.327 32 32z" class=""></path></svg>

		<svg aria-hidden="true" focusable="false" data-prefix="fas" data-icon="spinner" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" class="happyforms-address-geolocate__spinner"><path fill="currentColor" d="M304 48c0 26.51-21.49 48-48 48s-48-21.49-48-48 21.49-48 48-48 48 21.49 48 48zm-48 368c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48zm208-208c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48zM96 256c0-26.51-21.49-48-48-48S0 229.49 0 256s21.49 48 48 48 48-21.49 48-48zm12.922 99.078c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48c0-26.509-21.491-48-48-48zm294.156 0c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48c0-26.509-21.49-48-48-48zM108.922 60.922c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.491-48-48-48z" class=""></path></svg>
	</button>
	<?php
}

endif;

if ( ! function_exists( 'happyforms_phone_allows_multiple_conventions' ) ) :

function happyforms_phone_allow_multiple_conventions( $part ) {
	$allow_multiple = false;

	if ( 0 == $part['mask_allow_only_this_convention'] ) {
		$allow_multiple = true;
	}

	/**
	 * Handle back compatibility with previous control labeled "Allow all conventions".
	 */
	if ( isset( $part['mask_allow_all_countries'] ) ) {
		if ( 1 == $part['mask_allow_all_countries'] && 1 != $part['mask_allow_only_this_convention'] ) {
			$allow_multiple = true;
		} else {
			$allow_multiple = false;
		}
	}

	return $allow_multiple;
}

endif;
