<?php

class HappyForms_Service_AWeber extends HappyForms_Service {

	public $id = 'aweber';
	public $group = 'email';

	public $endpoint_authorize = 'https://auth.aweber.com/oauth2/authorize';
	public $endpoint_tokens = 'https://auth.aweber.com/oauth2/token';
	public $endpoint_account = 'https://api.aweber.com/1.0/accounts';
	public $endpoint_lists = 'https://api.aweber.com/1.0/accounts/%s/lists';
	public $endpoint_fields = 'https://api.aweber.com/1.0/accounts/%s/lists/%s/custom_fields';
	public $endpoint_subscriber = 'https://api.aweber.com/1.0/accounts/%s/lists/%s/subscribers';

	public $redirect_uri = '';
	public $scopes = 'account.read list.read list.write subscriber.write';
	public $state_nonce = 'happyforms_aweber_auth';
	public $oauth_action = 'happyforms-oauth';

	public $lists = null;
	public $fields = null;

	public function __construct() {
		$this->label = __( 'AWeber', 'happyforms' );
		$this->redirect_uri = add_query_arg( array(
			$this->oauth_action => $this->id,
		), admin_url() );

		add_action( 'admin_init', array( $this, 'handle_authorize_redirect' ) );
	}

	public function get_default_credentials() {
		$credentials = array(
			'client_id' => '',
			'client_secret' => '',
			'access_token' => '',
			'refresh_token' => '',
			'expires_in' => '',
			'last_updated' => '',
			'account_id' => '',
		);

		return $credentials;
	}

	public function set_credentials( $credentials = array(), $raw = array() ) {
		$this->credentials = wp_parse_args( $credentials, $this->get_default_credentials() );
	}

	public function admin_widget( $previous_credentials = array() ) {
		if ( wp_doing_ajax() && $this->credentials_changed( $previous_credentials ) ) {
			wp_send_json( array(
				'url' => $this->get_authorize_url(),
			) );
		}

		require_once( happyforms_get_integrations_folder() . '/services/aweber/partial-widget.php' );
	}

	public function credentials_changed( $previous_credentials ) {
		if ( $this->credentials['client_id'] !== $previous_credentials['client_id'] ) {
			return true;
		}

		if ( $this->credentials['client_secret'] !== $previous_credentials['client_secret'] ) {
			return true;
		}

		return false;
	}

	public function is_connected() {
		$is_connected = ! empty( $this->credentials['access_token'] );

		return $is_connected;
	}

	public function load() {
		if ( $this->is_connected() ) {
			require_once( happyforms_get_integrations_folder() . '/services/aweber/class-integration-aweber.php' );
		}
	}

	public function set_tokens( $access_token, $refresh_token ) {
		$this->access_token = $access_token;
		$this->refresh_token = $refresh_token;
	}

	public function get_authorize_url() {
		$state = wp_create_nonce( $this->state_nonce );
		$client_id = $this->credentials['client_id'];

		$url = add_query_arg( array(
			'response_type' => 'code',
			'client_id' => $client_id,
			'redirect_uri' => $this->redirect_uri,
			'scope' => $this->scopes,
			'state' => $state,
		), $this->endpoint_authorize );

		return $url;
	}

	public function handle_authorize_redirect() {
		if ( ! isset( $_GET[$this->oauth_action] ) || $_GET[$this->oauth_action] !== $this->id ) {
			return;
		}

		if ( ! isset( $_GET['code'] ) ) {
			return;
		}

		$authorization_code = $_GET['code'];
		$tokens = $this->get_tokens( $authorization_code );
		$this->credentials = wp_parse_args( $tokens, $this->credentials );
		$this->credentials['last_updated'] = time();
		$this->credentials['account_id'] = $this->get_account_id();

		happyforms_get_integrations()->write_credentials();

		$redirect_url = admin_url( 'admin.php?page=happyforms-settings' );

		wp_redirect( $redirect_url );
	}

	public function get_tokens( $authorization_code ) {
		$client_id = $this->credentials['client_id'];
		$client_secret = $this->credentials['client_secret'];

		$response = wp_remote_post( $this->endpoint_tokens, array(
			'body' => array(
				'grant_type' => 'authorization_code',
				'client_id' => $client_id,
				'client_secret' => $client_secret,
				'code' => $authorization_code,
				'redirect_uri' => $this->redirect_uri,
			)
		) );

		$body = wp_remote_retrieve_body( $response );
		$body = json_decode( $body, true );

		return $body;
	}

	public function get_refresh_tokens() {
		$client_id = $this->credentials['client_id'];
		$client_secret = $this->credentials['client_secret'];
		$refresh_token = $this->credentials['refresh_token'];

		$response = wp_remote_post( $this->endpoint_tokens, array(
			'body' => array(
				'grant_type' => 'refresh_token',
				'client_id' => $client_id,
				'client_secret' => $client_secret,
				'refresh_token' => $refresh_token,
			)
		) );

		$body = wp_remote_retrieve_body( $response );
		$body = json_decode( $body, true );

		return $body;
	}

	public function tokens_expired() {
		$now = time();
		$expires_in = intval( $this->credentials['expires_in'] );
		$last_updated = intval( $this->credentials['last_updated'] );
		$expired = $now - $last_updated >= $expires_in;

		return $expired;
	}

	public function refresh_tokens() {
		$tokens = $this->get_refresh_tokens();
		$this->credentials = wp_parse_args( $tokens, $this->credentials );
		$this->credentials['last_updated'] = time();

		happyforms_get_integrations()->write_credentials();
	}

	public function make_request( $method, $enpoint, $body = false ) {
		if ( $this->tokens_expired() ) {
			$this->refresh_tokens();
		}

		$access_token = $this->credentials['access_token'];
		$arguments = array(
			'method' => $method,
			'headers' => array(
				'Authorization' => 'Bearer ' . $access_token,
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			)
		);

		if ( $body ) {
			$arguments['body'] = json_encode( $body );
		}

		$response = wp_remote_request( $enpoint, $arguments );
		$code = wp_remote_retrieve_response_code( $response );
		$response = wp_remote_retrieve_body( $response );
		$response = json_decode( $response );

		if ( 200 !== $code ) {
			$message = '';

			if ( isset( $response->error ) && isset( $response->error->message ) ) {
				$message = $response->error->message;
			}

			$response = new WP_Error( $this->id, $message );
		}

		if ( is_wp_error( $response ) && happyforms_debug_log_enabled() ) {
			happyforms_log_error( $response );
		}

		return $response;
	}

	public function get_account_id() {
		$response = $this->make_request( 'GET', $this->endpoint_account );

		if ( ! isset( $response->entries ) || 1 > count( $response->entries ) ) {
			$account_id = '';

			return $account_id;
		}

		$account_id = $response->entries[0]->id;

		return $account_id;
	}

	public function get_lists() {
		if ( ! is_null( $this->lists ) ) {
			return $this->lists;
		}

		$account_id = $this->credentials['account_id'];
		$endpoint = sprintf( $this->endpoint_lists, $account_id );
		$response = $this->make_request( 'GET', $endpoint );
		$this->lists = array();

		if ( is_wp_error( $response ) ) {
			return $this->lists;
		}

		$this->lists = array_map( function( $list ) {
			$list = array(
				'id' => $list->id,
				'name' => $list->name,
			);

			return $list;
		}, $response->entries );

		return $this->lists;
	}

	public function get_standard_fields() {
		$fields = array( array(
			'id' => 'email',
			'name' => __( 'Email', 'happyforms' ),
		), array(
			'id' => 'name',
			'name' => __( 'Full name', 'happyforms' ),
		) );

		return $fields;
	}

	public function get_fields() {
		if ( ! is_null( $this->fields ) ) {
			return $this->fields;
		}

		$account_id = $this->credentials['account_id'];
		$lists = $this->get_lists();
		$fields = array();

		foreach( $lists as $list ) {
			$list_id = $list['id'];
			$endpoint = sprintf( $this->endpoint_fields, $account_id, $list_id );
			$response = $this->make_request( 'GET', $endpoint );

			$list_fields = array_map( function( $field ) {
				$field = array(
					'id' => $field->name,
					'name' => $field->name,
				);

				return $field;
			}, $response->entries );

			$this->fields[$list_id] = array_merge( $list_fields, $this->get_standard_fields() );
		}

		return $this->fields;
	}

	public function add_subscriber( $list_id, $fields, $tags = array() ) {
		$account_id = $this->credentials['account_id'];
		$endpoint = sprintf( $this->endpoint_subscriber, $account_id, $list_id );
		$standard_fields = wp_list_pluck( $this->get_standard_fields(), 'name', 'id' );
		$subscriber = array_intersect_key( $fields, $standard_fields );
		$custom_fields = array_diff_key( $fields, $subscriber );

		if ( ! empty( array_keys( $custom_fields ) ) ) {
			$subscriber['custom_fields'] = $custom_fields;
		}

		if ( ! empty( $tags ) ) {
			$subscriber['tags'] = $tags;
		}

		$response = $this->make_request( 'POST', $endpoint, $subscriber );
	}

}
